
/**
 * Declares constants for AST tags.
 *
 * @author    <a href="mailto:hj@janicke.de">hj</a>
 * @created   26. Juni 2002
 */
public interface ASTConstants
{

	/**
	 * Tag for binary operators. A binary node.<br> 
	 * child[0] is left operand<br>
	 * child[1] is right operand<br>
	 */
	public final static int AST_BINARY_OP = 0;

	/**
	 * Tag for unary operators. A unary node. <br>
	 * child[0] is operand<br>
	 */
	public final static int AST_UNARY_OP = 1;

	/**
	 * Tag for ternary operators. A ternary node.<br>
	 * Not used yet.
	 */
	public final static int AST_TERNARY_OP = 2;

	/** 
	 * Tag for statement list.<br>
	 * childCount is no of statements.  
	 */
	public final static int AST_STATEMENT_LIST = 3;

	/**
	 * Tag for program. A ternary node. <br>
	 * child[0] is program name <br>
	 * child[1] is declaration list <br>
	 * child[2] is statement list (run)
	 */
	public final static int AST_PROGRAM = 4;

	/** Tag for identifier. Terminal node.  */
	public final static int AST_IDENTIFIER = 5;

	/**
	 * Tag for declaration list.<br>
	 * childCount is no of declarations.
	 */
	public final static int AST_DECLARATION_LIST = 6;

	/**
	 * Tag for declaration.<br>
	 * child[0] .. child[n-1] are identifiers (varnames)<br> 
	 * child[n]               is identifier (typedenoter)
	 */
	public final static int AST_DECLARATION = 7;

	/**
	 * Tag for assignment. A binary node.<br>
	 * child[0] is identifier (varname)<br>
	 * child[1] is expression
	 */
	public final static int AST_ASSIGN = 10;

	/**
	 * Tag for IF THEN ELSE. A ternary node. <br>
	 * child[0] is expression (condition)<br>
	 * child[1] is statement  (then part)<br> 
	 * child[2] is statement  (else part)<br>
	 */
	public final static int AST_IF_THEN_ELSE = 11;

	/**
	 * Tag for FOR TO DO. A quaternary node.<br>
	 * child[0] is identifier (loop var)<br>
	 * child[1] is expression (initial value)<br>
	 * child[2] is expression (last value)<br>
	 * child[3] is statement (loop body)
	 */
	public final static int AST_FOR_TO_DO = 12;

	/**
	 * Tag for WHILE DO. A binary node.<br>
	 * child[0] is expression (condition)<br>
	 * child[1] is statement (loop body)
	 */
	public final static int AST_WHILE_DO = 13;

	/** Tag for boolean literal. Terminal node.  */
	public final static int AST_BOOLEAN_LITERAL = 16;

	/** Tag for integer literal. Terminal node.  */
	public final static int AST_INT_LITERAL = 17;

}

