import javax.swing.tree.*;

/**
 * An ASTNode is one node in the (A)bstract (S)yntax (T)ree. An ASTNodes
 * content is identified by its <i>astTag</i> . The corresponding Token
 * for this node can be optained using the <code>getToken()</code>
 * method.
 *
 * @author    <a href="mailto:hj@janicke.de">hj</a>
 * @created   26. Juni 2002
 * @see       ASTConstants
 */
public class ASTNode extends DefaultMutableTreeNode
{

	/**
	 * ObjectDescripor used for contextual analysis and code generation.
	 */
	protected ObjectDescriptor descriptor;

	/**
	 * AST tag is one of the constants defined in ASTConstants
	 *
	 * @see   ASTConstants
	 */
	protected int astTag;

	/** The token corresponding to this node. */
	protected TokenFactory.Token token;


	/**
	 * Constructs a new ASTNode.
	 *
	 * @param token   The token.
	 * @param astTag  The AST tag.
	 */
	public ASTNode(TokenFactory.Token token, int astTag) {
		setToken(token);
		setAstTag(astTag);
	}


	/**
	 * Sets the descriptor attribute of the ASTNode object
	 *
	 * @param descriptor  The new descriptor value
	 */
	public void setDescriptor(ObjectDescriptor descriptor) {
		this.descriptor = descriptor;
	}


	/**
	 * Sets the astTag attribute of the ASTNode object
	 *
	 * @param astTag  The new astTag value
	 */
	public void setAstTag(int astTag) {
		this.astTag = astTag;
	}


	/**
	 * Sets the token attribute of the ASTNode object
	 *
	 * @param token  The new token value
	 */
	public void setToken(TokenFactory.Token token) {
		this.token = token;
	}


	/**
	 * Gets the descriptor attribute of the ASTNode object
	 *
	 * @return   The descriptor value
	 */
	public ObjectDescriptor getDescriptor() {
		return descriptor;
	}


	/**
	 * Gets the astTag attribute of the ASTNode object
	 *
	 * @return   The astTag value
	 */
	public int getAstTag() {
		return astTag;
	}


	/**
	 * Gets the token attribute of the ASTNode object
	 *
	 * @return   The token value
	 */
	public TokenFactory.Token getToken() {
		return token;
	}


	/**
	 * Returns a String representation of this node
	 *
	 * @return   String description
	 */
	public String toString() {
		return "ASTTag = " + getAstTag() + "   " + ((token == null) ? null : token.getSpelling());
	}
}

