import java.util.*;
import java.io.*;

/** Runs the 4 compilation steps.<br>
 *  1) Lexical analysis.
 *  2) Syntactical analysis.
 *  3) Contextional analysis.
 *  4) Code generation.
 */
public class CompilerDriver {
	
	protected LexicalAnalyzer lexicalAnalyzer = new DefaultLexicalAnalyzer();
	protected SyntaxAnalyzer  syntaxAnalyzer  = new DefaultSyntaxAnalyzer();
//	protected ContextAnalyzer contextAnalyzer;
//	protected CodeGenerator   codeGenerator;
	
	/** Compiles the file.
	 *  @param f The file to compile.
	 *  @return 0 if succeeded.
	 */
	public int compile(File f) throws IOException {
		int rv = 1;
		try {
			InputStream in  = new FileInputStream(f);
			List tokens     = lexicalAnalyzer.tokenize(in);
			Iterator tok = tokens.iterator();
			while (tok.hasNext()) System.err.println(tok.next());
			
			
			syntaxAnalyzer.setTokenList(tokens);
			ASTNode ast     = syntaxAnalyzer.parseProgram(); 
			rv = 0; // succeeded.
			
			// tests only
			TestGUI gui = new TestGUI();
			gui.setTokenList(tokens);
			gui.setAST(ast);
			gui.show();
		} catch (LexicalException lexc) {
			lexc.printStackTrace();
		} catch (SyntaxException sexc) {
			sexc.printStackTrace();
		}
		
		
		return rv;
	}
	
	// Tester.
	public static void main(String[] args) {
		CompilerDriver compiler = new CompilerDriver();
		
		if (args.length > 0) {
			for (int i=0; i<args.length; i++) {
				File f = new File(args[i]);
				if (!f.exists() || !f.canRead()) {
					System.out.println("cannot read file: " + f);
					continue;
				} else {
					try {
						compiler.compile(f);
					} catch(IOException exc) {
						System.out.println("IO error compiling: " + f);
						exc.printStackTrace();
					}
				}
				
			}
		} else {
			System.out.println("not enough arguments:\nusage: java CompilerDriver file1 file2 ...");
		}
	}
}
