/** Generates Token objects from spelling. */
public class TokenFactory implements TOKConstants {
	
	/** Returns an EOT Token. */
	public static Token createEOT(int lineno) {
		return new Token("EOT", TOK_EOT, lineno);
	}
	
	/** Returns an Token for the spelling, having the token type defined in
	 *  specification.
	 *  @see TOKConstants
	 */
	public static Token createToken(String spelling, int lineno) throws LexicalException {
		int type = -1;
		if (spelling.equalsIgnoreCase("+")) {
			type = TOK_PLUS;
		} else
		if (spelling.equalsIgnoreCase("++")) {
			type = TOK_INCREMENT_1;
		} else
		if (spelling.equalsIgnoreCase("+=")) {
			type = TOK_INCREMENT_X;
		} else
		if (spelling.equalsIgnoreCase("-")) {
			type = TOK_MINUS;
		} else
		if (spelling.equalsIgnoreCase("--")) {
			type = TOK_DECREMENT_1;
		} else
		if (spelling.equalsIgnoreCase("-=")) {
			type = TOK_DECREMENT_X;
		} else
		if (spelling.equalsIgnoreCase("*")) {
			type = TOK_MUL;
		} else
		if (spelling.equalsIgnoreCase("/")) {
			type = TOK_DIV;
		} else
		if (spelling.equalsIgnoreCase("PROGRAM")) {
			type = TOK_PROGRAM;
		} else
		if (spelling.equalsIgnoreCase("DECLARE")) {
			type = TOK_DECLARE;
		} else
		if (spelling.equalsIgnoreCase("{")) {
			type = TOK_BLOCKBEGIN;
		} else
		if (spelling.equalsIgnoreCase("}")) {
			type = TOK_BLOCKEND;
		} else
		if (spelling.equalsIgnoreCase("WHILE")) {
			type = TOK_WHILE;
		} else
		if (spelling.equalsIgnoreCase("DO")) {
			type = TOK_DO;
		} else
		if (spelling.equalsIgnoreCase("FOR")) {
			type = TOK_FOR;
		} else
		if (spelling.equalsIgnoreCase("TO")) {
			type = TOK_TO;
		} else
		if (spelling.equalsIgnoreCase("DOWNTO")) {
			type = TOK_DOWNTO;
		} else
		if (spelling.equalsIgnoreCase("STEP")) {
			type = TOK_STEP;
		} else
		if (spelling.equalsIgnoreCase("IF")) {
			type = TOK_IF;
		} else
		if (spelling.equalsIgnoreCase("THEN")) {
			type = TOK_THEN;
		} else
		if (spelling.equalsIgnoreCase("ELSE")) {
			type = TOK_ELSE;
		} else
		if (spelling.equalsIgnoreCase("TYPE")) {
			type = TOK_TYPE;
		} else
		if (spelling.equalsIgnoreCase("RECORD")) {
			type = TOK_RECORD;
		} else
		if (spelling.equalsIgnoreCase("=")) {
			type = TOK_ASSIGN;
		} else
		if (spelling.equalsIgnoreCase("==")) {
			type = TOK_EQ;
		} else
		if (spelling.equalsIgnoreCase("!=")) {
			type = TOK_NEQ;
		} else
		if (spelling.equalsIgnoreCase(">")) {
			type = TOK_GT;
		} else
		if (spelling.equalsIgnoreCase("<")) {
			type = TOK_LT;
		} else
		if (spelling.equalsIgnoreCase(">=")) {
			type = TOK_GTE;
		} else
		if (spelling.equalsIgnoreCase("<=")) {
			type = TOK_LTE;
		} else
		if (spelling.equalsIgnoreCase("&&")) {
			type = TOK_CONDAND;
		} else
		if (spelling.equalsIgnoreCase("||")) {
			type = TOK_CONDOR;
		} else
		if (spelling.equalsIgnoreCase("&")) {
			type = TOK_AND;
		} else
		if (spelling.equalsIgnoreCase("|")) {
			type = TOK_OR;
		} else
		if (spelling.equalsIgnoreCase("!")) {
			type = TOK_NOT;
		} else
		if (spelling.equalsIgnoreCase("~")) {
			type = TOK_COMPLEMENT;
		} else
		if (spelling.equalsIgnoreCase(",")) {
			type = TOK_COMMA;
		} else
		if (spelling.equalsIgnoreCase(":")) {
			type = TOK_COLON;
		} else
		if (spelling.equalsIgnoreCase("TRUE")) {
			type = TOK_TRUE;
		} else
		if (spelling.equalsIgnoreCase("FALSE")) {
			type = TOK_FALSE;
		} else
		if (spelling.equalsIgnoreCase("RUN")) {
			type = TOK_RUN;
		} else
		if (spelling.equalsIgnoreCase(";")) {
			type = TOK_SEMICOLON;
		} else
		if (spelling.equalsIgnoreCase("(")) {
			type = TOK_OPEN_PAR;
		} else
		if (spelling.equalsIgnoreCase(")")) {
			type = TOK_CLOSE_PAR;
		} else
		if (Character.isDigit(spelling.charAt(0))) {
			type = TOK_INT_LITERAL;
		} else 
		if (Character.isJavaIdentifierStart(spelling.charAt(0))) {
			type = TOK_IDENTIFIER;
		} else {
			System.out.println("hello world");
			throw new LexicalException("unknown character sequence: \'" + spelling + "\'", lineno);
		}
		
		return new Token(spelling, type, lineno);
	}
	
	/** A token stores information of linenumber, spelling and Tokentype. */
	public static class Token {
		
		/** the spelling */
		protected String spelling;
		
		/** the ytpe @see TOKConstants */
		protected int type;
		
		/** the linenumber of the token. */
		protected long lineno;
	
		/** Constructs a new token.
		 *  @param spelling The spelling
		 *  @param type The tokens type.
		 *  @param lineno The tokens linenumber.
		 */
		protected Token(String spelling, int type, long lineno) {
			this.spelling = spelling;
			this.type     = type;
			this.lineno   = lineno;
		}
		
		/**
		 * Gets the spelling attribute of the Token object
		 *
		 * @return   The spelling value
		 */
		public String getSpelling() {
			return spelling;
		}


		/**
		 * Gets the type attribute of the Token object
		 *
		 * @return   The type value
		 */
		public int getType() {
			return type;
		}


		/**
		 * Gets the lineno attribute of the Token object
		 *
		 * @return   The lineno value
		 */
		public long getLineno() {
			return lineno;
		}
		
		/** Returns string descritpion of the token.
 		 *  @return description.
		 */
		public String toString() {
			return spelling + " [" + type + "] " + " line= " + lineno;	
		}
				
	} //eoc Token
	
} //eoc TokenFactory
